@@ -5,6 +5,7 @@
 
 #include <pl/helpers/utils.hpp>
 #include <pl/helpers/concepts.hpp>
+#include <pl/helpers/variant_type_index.hpp>
 #include <variant>
 
 namespace pl::core {
@@ -112,6 +113,9 @@ namespace pl::core {
 
     std::strong_ordering Token::Literal::operator<=>(const Literal &other) const {
         return std::visit(wolv::util::overloaded {
+            [](std::shared_ptr<ptrn::Pattern> lhs, std::shared_ptr<ptrn::Pattern> rhs) {
+                return lhs->getOffset() <=> rhs->getOffset();
+            },
             []<typename T>(T lhs, T rhs) -> std::strong_ordering {
                 if (lhs == rhs) return std::strong_ordering::equal;
                 if (lhs < rhs) return std::strong_ordering::less;
@@ -157,8 +161,8 @@ namespace pl::core {
                     return std::strong_ordering::greater;
                 }
             },
-            [](auto, auto) -> std::strong_ordering {
-                return std::strong_ordering::less;
+            []<typename TL, typename TR>(TL, TR) -> std::strong_ordering {
+                return hlp::VariantTypeIndex<TL, LiteralVariantType> <=> hlp::VariantTypeIndex<TR, LiteralVariantType>;
             }
         }, *this, other);
     }
