@@ -50,11 +50,20 @@ namespace pl {
     }
 
     PatternLanguage::~PatternLanguage() {
+        if (this->m_flattenThread.joinable())
+            this->m_flattenThread.join();
+        this->m_parserManager.reset();
+        if (this->m_internals.parser)
+            this->m_internals.parser->reset();
         this->m_patterns.clear();
         this->m_flattenedPatterns.clear();
+        this->m_flattenedPatternsValid = false;
     }
 
     PatternLanguage::PatternLanguage(PatternLanguage &&other) noexcept {
+        if (this->m_flattenThread.joinable())
+            this->m_flattenThread.join();
+
         this->m_internals           = std::move(other.m_internals);
         other.m_internals = { };
         this->m_internals.evaluator->setRuntime(this);
@@ -181,16 +190,13 @@ namespace pl {
         return m_currAST;
     }
 
-    bool PatternLanguage::executeString(std::string code, const std::string& source, const std::map<std::string, core::Token::Literal> &envVars, const std::map<std::string, core::Token::Literal> &inVariables, bool checkResult) {
+    bool PatternLanguage::executeString(const std::string& code, const std::string& source, const std::map<std::string, core::Token::Literal> &envVars, const std::map<std::string, core::Token::Literal> &inVariables, bool checkResult) {
 	   	const auto startTime = std::chrono::high_resolution_clock::now();
         ON_SCOPE_EXIT {
             const auto endTime = std::chrono::high_resolution_clock::now();
             this->m_runningTime = std::chrono::duration_cast<std::chrono::duration<double>>(endTime - startTime).count();
         };
 
-        code = wolv::util::replaceStrings(code, "\r\n", "\n");
-        code = wolv::util::replaceStrings(code, "\t", "    ");
-
         const auto &evaluator = this->m_internals.evaluator;
 
         evaluator->getConsole().setLogCallback(this->m_logCallback);
@@ -247,6 +253,7 @@ namespace pl {
             writeFunction
         );
 
+        evaluator->setStartAddress(this->getStartAddress());
         evaluator->setReadOffset(evaluator->getDataBaseAddress());
         evaluator->setDangerousFunctionCallHandler(this->m_dangerousFunctionCallCallback);
 
@@ -265,7 +272,7 @@ namespace pl {
                 if (node == nullptr)
                     continue;
 
-                auto location = node->getLocation();
+                const auto &location = node->getLocation();
                 if (lastLine == location.line)
                     continue;
 
@@ -275,7 +282,7 @@ namespace pl {
             }
 
             if (auto cursor = this->m_currError->cursorAddress; cursor.has_value())
-                console.log(core::LogConsole::Level::Error, fmt::format("Error happened with cursor at address 0x{:04X}", *cursor));
+                console.log(core::LogConsole::Level::Error, fmt::format("Error happened with cursor at address 0x{:04X}", *cursor + m_startAddress.value_or(0x00)));
 
             console.log(core::LogConsole::Level::Error, "\n");
         } else {
@@ -301,11 +308,11 @@ namespace pl {
             }
         }
 
-        this->flattenPatterns();
 
         if (this->m_aborted) {
             this->reset();
         } else {
+            this->flattenPatterns();
             this->m_patternsValid = true;
         }
 
@@ -334,6 +341,39 @@ namespace pl {
         return this->m_fileResolver.addVirtualFile(code, source, mainSource);
     }
 
+    std::multimap<std::string, std::string> PatternLanguage::getPragmaValues(const std::string &code, const std::string &source) const {
+        std::multimap<std::string, std::string> pragmaValues;
+
+        const api::Source plSource(code, source);
+        const auto result = m_internals.lexer->lex(&plSource);
+        if (result.isOk()) {
+            const auto tokens = result.unwrap();
+            for (auto it = tokens.begin(); it != tokens.end(); ++it) {
+                if (it->type == core::Token::Type::Directive && std::get<core::Token::Directive>(it->value) == core::Token::Directive::Pragma) {
+                    ++it;
+                    if (it != tokens.end() && it->type == core::Token::Type::String) {
+                        auto literal = std::get<core::Token::Literal>(it->value);
+                        auto string = std::get_if<std::string>(&literal);
+                        if (string != nullptr) {
+                            auto pragmaKey = *string;
+                            ++it;
+                            if (it != tokens.end() && it->type == core::Token::Type::String) {
+                                literal = std::get<core::Token::Literal>(it->value);
+                                string = std::get_if<std::string>(&literal);
+                                if (string != nullptr) {
+                                    pragmaValues.emplace(pragmaKey, *string);
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
+        return pragmaValues;
+    }
+
+
     void PatternLanguage::abort() {
         this->m_internals.evaluator->abort();
         this->m_aborted = true;
@@ -455,14 +495,20 @@ namespace pl {
 
 
     void PatternLanguage::reset() {
+        if (this->m_flattenThread.joinable())
+            this->m_flattenThread.join();
         this->m_patterns.clear();
         this->m_flattenedPatterns.clear();
+        this->m_flattenedPatternsValid = false;
 
         this->m_currError.reset();
         this->m_compileErrors.clear();
         this->m_parserManager.reset();
         this->m_internals.validator->setRecursionDepth(32);
 
+        this->m_internals.preprocessor->reset();
+        this->m_internals.lexer->reset();
+        this->m_internals.parser->reset();
         this->m_internals.evaluator->getConsole().clear();
         this->m_internals.evaluator->setDefaultEndian(this->m_defaultEndian);
         this->m_internals.evaluator->setEvaluationDepth(32);
@@ -499,13 +545,26 @@ namespace pl {
     }
 
     void PatternLanguage::flattenPatterns() {
-
         for (const auto &[section, patterns] : this->m_patterns) {
+            if (this->m_aborted)
+                return;
+
             auto &sectionTree = this->m_flattenedPatterns[section];
             for (const auto &pattern : patterns) {
-                auto children = pattern->getChildren();
+                if (this->m_aborted)
+                    return;
 
-                for (const auto &[address, child]: children) {
+                if (auto staticArray = dynamic_cast<ptrn::PatternArrayStatic*>(pattern.get()); staticArray != nullptr) {
+                    if (staticArray->getEntryCount() > 0 && staticArray->getEntry(0)->getChildren().empty()) {
+                        const auto address = staticArray->getOffset();
+                        const auto size = staticArray->getSize();
+                        sectionTree.insert({ address, address + size - 1 }, staticArray);
+                        continue;
+                    }
+                }
+
+                auto children = pattern->getChildren();
+                for (const auto &[address, child] : children) {
                     if (this->m_aborted)
                         return;
 
@@ -535,7 +594,7 @@ namespace pl {
 
             // Handle static array members
             if (parent != nullptr) {
-                parent->setOffset(interval.interval.start);
+                parent->setOffset(interval.interval.start - (value->getOffset() - parent->getOffset()));
                 parent->clearFormatCache();
                 value->clearFormatCache();
             }
